// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

// THIS FILE IS AUTO-GENERATED, PLEASE DO NOT MODIFY BY HAND.

part of molview_dataserver.messages.v0;

/// Searching messages
/// ==================
/// These messages should be used for extensive search operations such as
/// full-text searches, graph searches, or even SQL queries. There are also
/// messages to provide search autocomplete data for databases.

/// Announce the addition of a new autocomplete provider
class AnnounceAutocompleteProvider extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_dataserver.v0.AnnounceAutocompleteProvider';

  /// Unique provider name
  final String name;

  /// Constructor
  AnnounceAutocompleteProvider(this.name) : super(id);

  /// Construct from passive data structure.
  factory AnnounceAutocompleteProvider.fromPds(final Map<String, dynamic> pds) {
    String name;

    // Read name.
    if (pds.containsKey('name') && pds['name'] is String) {
      name = pds['name'] as String;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'name'");
    }

    return new AnnounceAutocompleteProvider(name);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'name': name};
  }
}

/// Message handler for AnnounceAutocompleteProvider
class AnnounceAutocompleteProviderHandler extends OmnibusEventHandler {
  /// Callback
  final OmnibusEventCallback<AnnounceAutocompleteProvider> _callback;

  /// Constructor
  AnnounceAutocompleteProviderHandler(this._callback, StateData state)
      : super(AnnounceAutocompleteProvider.id, state);

  /// Process incoming message.
  Future<List<Future<Null>>> process(OmnibusEvent data) async {
    AnnounceAutocompleteProvider message = data is AnnounceAutocompleteProvider
        ? data
        : new AnnounceAutocompleteProvider.fromPds(data.toPds());

    return [_callback(message, state)];
  }
}

/// Request autocomplete data for the given search string.
class FindSearchSuggestions extends OmnibusRequest {
  /// omnibusId
  static const id = 'molview_dataserver.v0.FindSearchSuggestions';

  /// Input text
  final String text;

  /// Target language (two letter ISO code)
  final String language;

  /// Number of desired results per category
  final int count;

  /// List of provider names that are enabled for this request
  final List<String> accept;

  /// Constructor
  FindSearchSuggestions(this.text, this.language, this.accept,
      [this.count = 10])
      : super(id);

  /// Construct from passive data structure.
  factory FindSearchSuggestions.fromPds(final Map<String, dynamic> pds) {
    String text;
    String language;
    int count;
    List<String> accept;

    // Read text.
    if (pds.containsKey('text') && pds['text'] is String) {
      text = pds['text'] as String;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'text'");
    }

    // Read language.
    if (pds.containsKey('language') && pds['language'] is String) {
      language = pds['language'] as String;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'language'");
    }

    // Read count.
    if (pds.containsKey('count') && pds['count'] is int) {
      count = pds['count'] as int;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'count'");
    }

    // Read accept.
    if (pds.containsKey('accept') && pds['accept'] is List) {
      accept = new List<String>.from(pds['accept']);
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'accept'");
    }

    return new FindSearchSuggestions(text, language, accept, count);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {
      'text': text,
      'language': language,
      'count': count,
      'accept': accept
    };
  }
}

/// Message handler for FindSearchSuggestions
class FindSearchSuggestionsHandler extends OmnibusRequestHandler {
  /// Callback
  final OmnibusRequestCallback<FindSearchSuggestions> _callback;

  /// Constructor
  FindSearchSuggestionsHandler(this._callback, StateData state)
      : super(FindSearchSuggestions.id, state);

  /// Process incoming message.
  Future<List<Future<OmnibusResponse>>> process(OmnibusRequest data) async {
    FindSearchSuggestions message = data is FindSearchSuggestions
        ? data
        : new FindSearchSuggestions.fromPds(data.toPds());

    return [_callback(message, state)];
  }
}

/// Return a list of autocomplete suggestions.
class SearchSuggestions extends OmnibusResponse {
  /// omnibusId
  static const id = 'molview_dataserver.v0.SearchSuggestions';

  /// Provider name
  final String provider;

  /// List of autocomplete suggestions
  final List<SearchSuggestion> suggestions;

  /// Constructor
  SearchSuggestions(this.provider, this.suggestions) : super(id);

  /// Construct from passive data structure.
  factory SearchSuggestions.fromPds(final Map<String, dynamic> pds) {
    String provider;
    List<SearchSuggestion> suggestions;

    // Read provider.
    if (pds.containsKey('provider') && pds['provider'] is String) {
      provider = pds['provider'] as String;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'provider'");
    }

    // Read suggestions.
    if (pds.containsKey('suggestions') && pds['suggestions'] is List) {
      suggestions = pdsToList(
          pds['suggestions'],
          (Map<String, dynamic> item) => new SearchSuggestion.fromPds(
              new Map<String, dynamic>.from(item))) as List<SearchSuggestion>;
    } else {
      throw new ArgumentError.value(pds, 'pds',
          "does not contain a valid field for property 'suggestions'");
    }

    return new SearchSuggestions(provider, suggestions);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'provider': provider, 'suggestions': pdsFromList(suggestions)};
  }
}

/// Create SearchSuggestions Stream from the given responses.
Stream<SearchSuggestions> streamSearchSuggestions(OmnibusResponses responses) {
  return responses.stream(
          SearchSuggestions.id,
          (OmnibusResponse data) => data is SearchSuggestions
              ? data
              : new SearchSuggestions.fromPds(data.toPds()))
      as Stream<SearchSuggestions>;
}
