// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

// THIS FILE IS AUTO-GENERATED, PLEASE DO NOT MODIFY BY HAND.

part of molview_dataserver.messages.v0;

/// DataServer messages
/// ===================
/// The DataServer is the central, in-memory data storage in MolView. The
/// DataServer manages the retrieval of data records from external databases and
/// web-services. It can also be used to integrate local resources with external
/// resources. Modules can submit new data tables to the DataServer. The
/// DataServer takes care of interconnecting tables and finding the best way to
/// get the data that is requested by modules. To do so the DataServer stores a
/// map of keys that link to the most recent data in the application. These keys
/// can be used to retrieve related data.

/// Announce the addition of a new resource table.
class AnnounceTable extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_dataserver.v0.AnnounceTable';

  /// Database name
  final String database;

  /// Table name
  final String table;

  /// Table information
  final TableDescriptor descriptor;

  /// Constructor
  AnnounceTable(this.database, this.table, this.descriptor) : super(id);

  /// Construct from passive data structure.
  factory AnnounceTable.fromPds(final Map<String, dynamic> pds) {
    String database;
    String table;
    TableDescriptor descriptor;

    // Read database.
    if (pds.containsKey('database') && pds['database'] is String) {
      database = pds['database'] as String;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'database'");
    }

    // Read table.
    if (pds.containsKey('table') && pds['table'] is String) {
      table = pds['table'] as String;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'table'");
    }

    // Read descriptor.
    if (pds.containsKey('descriptor') && pds['descriptor'] is Map) {
      descriptor = new TableDescriptor.fromPds(
          new Map<String, dynamic>.from(pds['descriptor']));
    } else {
      throw new ArgumentError.value(pds, 'pds',
          "does not contain a valid field for property 'descriptor'");
    }

    return new AnnounceTable(database, table, descriptor);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {
      'database': database,
      'table': table,
      'descriptor': descriptor.toPds()
    };
  }
}

/// Message handler for AnnounceTable
class AnnounceTableHandler extends OmnibusEventHandler {
  /// Callback
  final OmnibusEventCallback<AnnounceTable> _callback;

  /// Database name
  final String database;

  /// Table name
  final String table;

  /// Constructor
  AnnounceTableHandler(this._callback, StateData state,
      {this.database: null, this.table: null})
      : super(AnnounceTable.id, state);

  /// Process incoming message.
  Future<List<Future<Null>>> process(OmnibusEvent data) async {
    AnnounceTable message =
        data is AnnounceTable ? data : new AnnounceTable.fromPds(data.toPds());

    if ((database == null || database == message.database) &&
        (table == null || table == message.table)) {
      return [_callback(message, state)];
    } else {
      return [];
    }
  }
}

/// A table was announced and rejected
class TableRejected extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_dataserver.v0.TableRejected';

  /// Rejection reason
  final TableRejectionReason reason;

  /// User friendly rejection message
  final String message;

  /// Constructor
  TableRejected(this.reason, this.message) : super(id);

  /// Construct from passive data structure.
  factory TableRejected.fromPds(final Map<String, dynamic> pds) {
    TableRejectionReason reason;
    String message;

    // Read reason.
    if (pds.containsKey('reason') && pds['reason'] is int) {
      reason = TableRejectionReason.values[pds['reason']];
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'reason'");
    }

    // Read message.
    if (pds.containsKey('message') && pds['message'] is String) {
      message = pds['message'] as String;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'message'");
    }

    return new TableRejected(reason, message);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'reason': reason.index, 'message': message};
  }
}

/// Message handler for TableRejected
class TableRejectedHandler extends OmnibusEventHandler {
  /// Callback
  final OmnibusEventCallback<TableRejected> _callback;

  /// Rejection reason
  final TableRejectionReason reason;

  /// Constructor
  TableRejectedHandler(this._callback, StateData state, {this.reason: null})
      : super(TableRejected.id, state);

  /// Process incoming message.
  Future<List<Future<Null>>> process(OmnibusEvent data) async {
    TableRejected message =
        data is TableRejected ? data : new TableRejected.fromPds(data.toPds());

    if ((reason == null || reason == message.reason)) {
      return [_callback(message, state)];
    } else {
      return [];
    }
  }
}

/// A table was announced and registered
class TableRegistered extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_dataserver.v0.TableRegistered';

  /// Database name
  final String database;

  /// Table name
  final String table;

  /// Resource table details
  final TableDescriptor descriptor;

  /// Constructor
  TableRegistered(this.database, this.table, this.descriptor) : super(id);

  /// Construct from passive data structure.
  factory TableRegistered.fromPds(final Map<String, dynamic> pds) {
    String database;
    String table;
    TableDescriptor descriptor;

    // Read database.
    if (pds.containsKey('database') && pds['database'] is String) {
      database = pds['database'] as String;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'database'");
    }

    // Read table.
    if (pds.containsKey('table') && pds['table'] is String) {
      table = pds['table'] as String;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'table'");
    }

    // Read descriptor.
    if (pds.containsKey('descriptor') && pds['descriptor'] is Map) {
      descriptor = new TableDescriptor.fromPds(
          new Map<String, dynamic>.from(pds['descriptor']));
    } else {
      throw new ArgumentError.value(pds, 'pds',
          "does not contain a valid field for property 'descriptor'");
    }

    return new TableRegistered(database, table, descriptor);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {
      'database': database,
      'table': table,
      'descriptor': descriptor.toPds()
    };
  }
}

/// Message handler for TableRegistered
class TableRegisteredHandler extends OmnibusEventHandler {
  /// Callback
  final OmnibusEventCallback<TableRegistered> _callback;

  /// Database name
  final String database;

  /// Table name
  final String table;

  /// Constructor
  TableRegisteredHandler(this._callback, StateData state,
      {this.database: null, this.table: null})
      : super(TableRegistered.id, state);

  /// Process incoming message.
  Future<List<Future<Null>>> process(OmnibusEvent data) async {
    TableRegistered message = data is TableRegistered
        ? data
        : new TableRegistered.fromPds(data.toPds());

    if ((database == null || database == message.database) &&
        (table == null || table == message.table)) {
      return [_callback(message, state)];
    } else {
      return [];
    }
  }
}

/// Request records from a table
class RetrieveData extends OmnibusRequest {
  /// omnibusId
  static const id = 'molview_dataserver.v0.RetrieveData';

  /// Target database
  final String database;

  /// Target table
  final String table;

  /// Input key
  final FieldData inkey;

  /// Retrieval parameters
  final Map<String, String> parameters;

  /// Requested keys by their index
  final List<int> keys;

  /// Requested properties by their index
  final List<int> properties;

  /// Constructor
  RetrieveData(this.database, this.table, this.inkey,
      {this.parameters: const {},
      this.keys: const [],
      this.properties: const []})
      : super(id);

  /// Construct from passive data structure.
  factory RetrieveData.fromPds(final Map<String, dynamic> pds) {
    String database;
    String table;
    FieldData inkey;
    Map<String, String> parameters;
    List<int> keys;
    List<int> properties;

    // Read database.
    if (pds.containsKey('database') && pds['database'] is String) {
      database = pds['database'] as String;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'database'");
    }

    // Read table.
    if (pds.containsKey('table') && pds['table'] is String) {
      table = pds['table'] as String;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'table'");
    }

    // Read inkey.
    if (pds.containsKey('inkey') && pds['inkey'] is Map) {
      inkey =
          new FieldData.fromPds(new Map<String, dynamic>.from(pds['inkey']));
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'inkey'");
    }

    // Read parameters.
    if (pds.containsKey('parameters') && pds['parameters'] is Map) {
      parameters = new Map<String, String>.from(pds['parameters']);
    } else {
      throw new ArgumentError.value(pds, 'pds',
          "does not contain a valid field for property 'parameters'");
    }

    // Read keys.
    if (pds.containsKey('keys') && pds['keys'] is List) {
      keys = new List<int>.from(pds['keys']);
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'keys'");
    }

    // Read properties.
    if (pds.containsKey('properties') && pds['properties'] is List) {
      properties = new List<int>.from(pds['properties']);
    } else {
      throw new ArgumentError.value(pds, 'pds',
          "does not contain a valid field for property 'properties'");
    }

    return new RetrieveData(database, table, inkey,
        parameters: parameters, keys: keys, properties: properties);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {
      'database': database,
      'table': table,
      'inkey': inkey.toPds(),
      'parameters': parameters,
      'keys': keys,
      'properties': properties
    };
  }
}

/// Message handler for RetrieveData
class RetrieveDataHandler extends OmnibusRequestHandler {
  /// Callback
  final OmnibusRequestCallback<RetrieveData> _callback;

  /// Target database
  final String database;

  /// Target table
  final String table;

  /// Constructor
  RetrieveDataHandler(this._callback, StateData state,
      {this.database: null, this.table: null})
      : super(RetrieveData.id, state);

  /// Process incoming message.
  Future<List<Future<OmnibusResponse>>> process(OmnibusRequest data) async {
    RetrieveData message =
        data is RetrieveData ? data : new RetrieveData.fromPds(data.toPds());

    if ((database == null || database == message.database) &&
        (table == null || table == message.table)) {
      return [_callback(message, state)];
    } else {
      return [];
    }
  }
}

/// Return retrieved data records to the DataServer
class DataRetrieved extends OmnibusResponse {
  /// omnibusId
  static const id = 'molview_dataserver.v0.DataRetrieved';

  /// Retrieved data records
  final List<DataRecord> records;

  /// Constructor
  DataRetrieved(this.records) : super(id);

  /// Construct from passive data structure.
  factory DataRetrieved.fromPds(final Map<String, dynamic> pds) {
    List<DataRecord> records;

    // Read records.
    if (pds.containsKey('records') && pds['records'] is List) {
      records = pdsToList(
              pds['records'],
              (Map<String, dynamic> item) =>
                  new DataRecord.fromPds(new Map<String, dynamic>.from(item)))
          as List<DataRecord>;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'records'");
    }

    return new DataRetrieved(records);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'records': pdsFromList(records)};
  }
}

/// Create DataRetrieved Stream from the given responses.
Stream<DataRetrieved> streamDataRetrieved(OmnibusResponses responses) {
  return responses.stream(
      DataRetrieved.id,
      (OmnibusResponse data) => data is DataRetrieved
          ? data
          : new DataRetrieved.fromPds(data.toPds())) as Stream<DataRetrieved>;
}

/// Send data to the DataServer
class SubmitData extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_dataserver.v0.SubmitData';

  /// Records that are sumbitted
  final List<DataRecord> records;

  /// Constructor
  SubmitData(this.records) : super(id);

  /// Construct from passive data structure.
  factory SubmitData.fromPds(final Map<String, dynamic> pds) {
    List<DataRecord> records;

    // Read records.
    if (pds.containsKey('records') && pds['records'] is List) {
      records = pdsToList(
              pds['records'],
              (Map<String, dynamic> item) =>
                  new DataRecord.fromPds(new Map<String, dynamic>.from(item)))
          as List<DataRecord>;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'records'");
    }

    return new SubmitData(records);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'records': pdsFromList(records)};
  }
}

/// Message handler for SubmitData
class SubmitDataHandler extends OmnibusEventHandler {
  /// Callback
  final OmnibusEventCallback<SubmitData> _callback;

  /// Constructor
  SubmitDataHandler(this._callback, StateData state)
      : super(SubmitData.id, state);

  /// Process incoming message.
  Future<List<Future<Null>>> process(OmnibusEvent data) async {
    SubmitData message =
        data is SubmitData ? data : new SubmitData.fromPds(data.toPds());

    return [_callback(message, state)];
  }
}

/// Find data snippets
class FindSnippets extends OmnibusRequest {
  /// omnibusId
  static const id = 'molview_dataserver.v0.FindSnippets';

  /// Requested data snippets
  final List<SnippetSelector> snippets;

  /// Alternative starting key
  final FieldData inkey;

  /// Constructor
  FindSnippets(this.snippets, {this.inkey: null}) : super(id);

  /// Construct from passive data structure.
  factory FindSnippets.fromPds(final Map<String, dynamic> pds) {
    List<SnippetSelector> snippets;
    FieldData inkey;

    // Read snippets.
    if (pds.containsKey('snippets') && pds['snippets'] is List) {
      snippets = pdsToList(
          pds['snippets'],
          (Map<String, dynamic> item) => new SnippetSelector.fromPds(
              new Map<String, dynamic>.from(item))) as List<SnippetSelector>;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'snippets'");
    }

    // Read inkey.
    if (pds.containsKey('inkey') && pds['inkey'] is Map) {
      inkey =
          new FieldData.fromPds(new Map<String, dynamic>.from(pds['inkey']));
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'inkey'");
    }

    return new FindSnippets(snippets, inkey: inkey);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'snippets': pdsFromList(snippets), 'inkey': inkey.toPds()};
  }
}

/// Message handler for FindSnippets
class FindSnippetsHandler extends OmnibusRequestHandler {
  /// Callback
  final OmnibusRequestCallback<FindSnippets> _callback;

  /// Constructor
  FindSnippetsHandler(this._callback, StateData state)
      : super(FindSnippets.id, state);

  /// Process incoming message.
  Future<List<Future<OmnibusResponse>>> process(OmnibusRequest data) async {
    FindSnippets message =
        data is FindSnippets ? data : new FindSnippets.fromPds(data.toPds());

    return [_callback(message, state)];
  }
}

/// Return snippets to the data consumer.
class SnippetsFound extends OmnibusResponse {
  /// omnibusId
  static const id = 'molview_dataserver.v0.SnippetsFound';

  /// Resulting snippets
  final List<SnippetData> snippets;

  /// Constructor
  SnippetsFound(this.snippets) : super(id);

  /// Construct from passive data structure.
  factory SnippetsFound.fromPds(final Map<String, dynamic> pds) {
    List<SnippetData> snippets;

    // Read snippets.
    if (pds.containsKey('snippets') && pds['snippets'] is List) {
      snippets = pdsToList(
              pds['snippets'],
              (Map<String, dynamic> item) =>
                  new SnippetData.fromPds(new Map<String, dynamic>.from(item)))
          as List<SnippetData>;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'snippets'");
    }

    return new SnippetsFound(snippets);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'snippets': pdsFromList(snippets)};
  }
}

/// Create SnippetsFound Stream from the given responses.
Stream<SnippetsFound> streamSnippetsFound(OmnibusResponses responses) {
  return responses.stream(
      SnippetsFound.id,
      (OmnibusResponse data) => data is SnippetsFound
          ? data
          : new SnippetsFound.fromPds(data.toPds())) as Stream<SnippetsFound>;
}

/// Try to switch to a new active record.
class PushActiveRecord extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_dataserver.v0.PushActiveRecord';

  /// One key that links to this record
  final FieldData key;

  /// Constructor
  PushActiveRecord(this.key) : super(id);

  /// Construct from passive data structure.
  factory PushActiveRecord.fromPds(final Map<String, dynamic> pds) {
    FieldData key;

    // Read key.
    if (pds.containsKey('key') && pds['key'] is Map) {
      key = new FieldData.fromPds(new Map<String, dynamic>.from(pds['key']));
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'key'");
    }

    return new PushActiveRecord(key);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'key': key.toPds()};
  }
}

/// Message handler for PushActiveRecord
class PushActiveRecordHandler extends OmnibusEventHandler {
  /// Callback
  final OmnibusEventCallback<PushActiveRecord> _callback;

  /// Constructor
  PushActiveRecordHandler(this._callback, StateData state)
      : super(PushActiveRecord.id, state);

  /// Process incoming message.
  Future<List<Future<Null>>> process(OmnibusEvent data) async {
    PushActiveRecord message = data is PushActiveRecord
        ? data
        : new PushActiveRecord.fromPds(data.toPds());

    return [_callback(message, state)];
  }
}

/// The active record is now pointing to a different record.
class ActiveRecordChanged extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_dataserver.v0.ActiveRecordChanged';

  /// Constructor
  ActiveRecordChanged() : super(id);

  /// Construct from passive data structure.
  factory ActiveRecordChanged.fromPds(final Map<String, dynamic> pds) {
    return new ActiveRecordChanged();
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {};
  }
}

/// Message handler for ActiveRecordChanged
class ActiveRecordChangedHandler extends OmnibusEventHandler {
  /// Callback
  final OmnibusEventCallback<ActiveRecordChanged> _callback;

  /// Constructor
  ActiveRecordChangedHandler(this._callback, StateData state)
      : super(ActiveRecordChanged.id, state);

  /// Process incoming message.
  Future<List<Future<Null>>> process(OmnibusEvent data) async {
    ActiveRecordChanged message = data is ActiveRecordChanged
        ? data
        : new ActiveRecordChanged.fromPds(data.toPds());

    return [_callback(message, state)];
  }
}
