// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of molview_dataserver.msglib;

/// Table property
class TableProperty extends TableField {
  /// Array property
  ///
  /// An array property can store multiple values of the specified data type.
  final bool array;

  /// Attributes map
  ///
  /// The attributes map allows you to extend fields beyond the limits of data
  /// classes. The attributes map can be used to find data fields that match
  /// certain requirements. For example, if you want moldata that is optimized
  /// using an MMFF94 energy minimization, you can search for a field that has
  /// this attribute in its attributes map.
  final Map<String, String> attributes;

  /// Constructor
  const TableProperty(
      DataType dataType, String dataClass, this.array, this.attributes)
      : super(dataType, dataClass);

  /// Construct from PDS.
  factory TableProperty.fromPds(Map<String, dynamic> data) {
    return new TableProperty(
        DataType.values[data['type'] as int],
        data['class'] as String,
        data['array'] as bool,
        data['attributes'] as Map<String, String>);
  }

  /// Convert to PDS.
  Map<String, dynamic> toPds() {
    return {
      'type': dataType.index,
      'class': dataClass,
      'array': array,
      'attributes': pdsFromMap(attributes)
    };
  }
}
