// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of molview_dataserver.standards;

/// Overview
/// ========
/// This is a list of all keys included in the standards library. A key should
/// meet at least one of the requirments listed below to be included in this
/// library.
/// - The key is used across multiple independent databases.
/// - The key is a general purpose identifier that can be generated using an
///   algorithm.

/// InChI
class InChI extends TableKey {
  static const String id = '${entityMolecule}_InChI';
  const InChI(KeyRelationType relation) : super(DataType.text, id, relation);
}

/// InChIKey
class InChIKey extends TableKey {
  static const String id = '${entityMolecule}_InChIKey';
  const InChIKey(KeyRelationType relation) : super(DataType.text, id, relation);
}

/// Canonical SMILES
class CanonicalSmiles extends TableKey {
  static const String id = '${entityMolecule}_CanonicalSmiles';
  const CanonicalSmiles(KeyRelationType relation)
      : super(DataType.text, id, relation);
}

/// Isomeric SMILES
class IsomericSmiles extends TableKey {
  static const String id = '${entityMolecule}_IsomericSmiles';
  const IsomericSmiles(KeyRelationType relation)
      : super(DataType.text, id, relation);
}

/// CAS Registry Number
class CasRN extends TableKey {
  static const String id = '${entitySubstance}_CasRN';
  const CasRN(KeyRelationType relation) : super(DataType.text, id, relation);
}

/// IUPAC name
class IupacName extends TableKey {
  static const String id = '${entityMolecule}_IupacName';
  const IupacName(KeyRelationType relation)
      : super(DataType.text, id, relation);
}

/// Protein Data Bank identifier code
class PdbId extends TableKey {
  static const String id = '${entityProtein}_PdbId';
  const PdbId(KeyRelationType relation) : super(DataType.text, id, relation);
}
