// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of molview_dataserver.standards;

/// Some common molecular properties. This list is based on the properties
/// provided by PubChem, see:
/// https://pubchem.ncbi.nlm.nih.gov/pug_rest/PUG_REST.html

/// Molecular formula
class MolecularFormula extends TableProperty {
  static const String id = '${entityMolecule}_MolecularFormula';
  const MolecularFormula(
      [bool array = false, Map<String, String> attributes = const {}])
      : super(DataType.text, id, array, attributes);
}

/// Molecular weight
class MolecularWeight extends TableProperty {
  static const String id = '${entityMolecule}_MolecularWeight';
  const MolecularWeight(
      [bool array = false, Map<String, String> attributes = const {}])
      : super(DataType.real, id, array, attributes);
}

/// XLogP
class XLogP extends TableProperty {
  static const String id = '${entityMolecule}_XLogP';
  const XLogP([bool array = false, Map<String, String> attributes = const {}])
      : super(DataType.real, id, array, attributes);
}

/// Exact mass
class ExactMass extends TableProperty {
  static const String id = '${entityMolecule}_ExactMass';
  const ExactMass(
      [bool array = false, Map<String, String> attributes = const {}])
      : super(DataType.real, id, array, attributes);
}

/// Monoisotopic mass
class MonoisotopicMass extends TableProperty {
  static const String id = '${entityMolecule}_MonoisotopicMass';
  const MonoisotopicMass(
      [bool array = false, Map<String, String> attributes = const {}])
      : super(DataType.real, id, array, attributes);
}

/// Topological Polar Surface Area
class TPSA extends TableProperty {
  static const String id = '${entityMolecule}_TPSA';
  const TPSA([bool array = false, Map<String, String> attributes = const {}])
      : super(DataType.real, id, array, attributes);
}

/// Complexity using the Bertz/Hendrickson/Ihlenfeldt formula
class Complexity extends TableProperty {
  static const String id = '${entityMolecule}_Complexity';
  const Complexity(
      [bool array = false, Map<String, String> attributes = const {}])
      : super(DataType.integer, id, array, attributes);
}

/// Electrical charge
class ElecticalCharge extends TableProperty {
  static const String id = '${entityMolecule}_ElecticalCharge';
  const ElecticalCharge(
      [bool array = false, Map<String, String> attributes = const {}])
      : super(DataType.real, id, array, attributes);
}

/// Number of hydrogen bond donors
class HBondDonorCount extends TableProperty {
  static const String id = '${entityMolecule}_HBondDonorCount';
  const HBondDonorCount(
      [bool array = false, Map<String, String> attributes = const {}])
      : super(DataType.integer, id, array, attributes);
}

/// Number of hydrogen bond acceptors
class HBondAcceptorCount extends TableProperty {
  static const String id = '${entityMolecule}_HBondAcceptorCount';
  const HBondAcceptorCount(
      [bool array = false, Map<String, String> attributes = const {}])
      : super(DataType.integer, id, array, attributes);
}

/// Number of rotatable bonds
class RotatableBondCount extends TableProperty {
  static const String id = '${entityMolecule}_RotatableBondCount';
  const RotatableBondCount(
      [bool array = false, Map<String, String> attributes = const {}])
      : super(DataType.integer, id, array, attributes);
}
